module.exports = function(grunt){
    var backendModules = ['../../teknisa/dc'];
    var directories = ['util', 'repositories', 'services', 'controllers'];
    var subModules = [];
    var libFiles = [
        'gitQueue/dist/main.min.js',

        'zh-grid-navigation/src/GridNavigationLibrary.js',
        'zh-interval-component/dist/zh-interval-component.js',

        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',
        'zh-grid-configuration/dist/zh-grid-configuration.js',

        'zh-memory-storage/src/*.js',
        "zh-cache-bypass/src/CacheBypass.js",

        'zhuaclfrontend/assets/dist/main.js',
        'zhulog/dist/main.js',

        'tek-libraries/dist/main.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zh-image-uploader/dist/zh-image-uploader.js'
    ];

    grunt.initConfig({
        concat: {
            js: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getJSPaths(),
                dest: './assets/js/app.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: './assets/css/app.css'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: './assets/js/libs.js'
            }
        },

        jshint: {
            all: ['Gruntfile.js', './assets/js/controllers/**/*.js', './assets/js/services/**/*.js', './assets/js/util/**/*.js']
        },

        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    './assets/js/app.min.js': getJSPaths()
                }
            }
        },

        watch: {
            pivotal: {
                files: getJSPaths().concat(["GruntFile.js"]),
                tasks: ['jshint', 'concat:js', 'concat:css']
            }
        },

        zhIdGenerator: {
            options: {
                jsonPath: getJSONPaths(),
                menuPath: ["./assets/json/menu.json"]
            }
        },

        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },

        buildEnvironment: {
            development: {
                configFile: "./config/devEnvironment.json"
            },
            production: {
                configFile: "./config/prodEnvironment.json"
            },
            test: {
                configFile: "./config/testEnvironment.json"
            },
            srcFolders: ["../backend/config/", "./assets/js/util", "./assets/json", "./config", "./", "./assets", "./assets/js"]
        }
    });

    function getJSONPaths() {
        var paths = ["./assets/json"];
        subModules.forEach(function (subModule) {
            paths.push("./assets/modules/" + subModule + "/assets/json");
        });
        return paths;
    }

    function getJSPaths() {
        var paths = [];
        directories.forEach(function (directory) {
            paths.push('./assets/js/' + directory + '/**/*.js');
            subModules.forEach(function (subModule) {
                paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
            });
        });
        return paths;
    }

    function getLibPaths() {
        var paths = [];
        libFiles.forEach(function(fileName) {
            paths.push('bower_components/' + fileName);
        });
        return paths;
    }

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/**/*.css');
        paths.push('!./assets/css/app.css');
        paths.push('bower_components/zh-preferences/dist/style/style.css');
        paths.push('bower_components/zh-image-uploader/dist/zh-image-uploader.css');
        return paths;
    }

    function buildFileContent(srcFiles, filePath, directory) {
        var parts = filePath.split('/');
        parts.shift();
        parts.shift();
        var filename = parts[parts.length - 1];
        var extension = filename.split('.').pop().toUpperCase();
        if (extension == 'JSON' || extension == 'XML') {
            if (directory == 'datasources') {
                filename = parts.pop();
                filename = filename.replace(/\.[^/.]+$/, "");
                srcFiles[directory][filename] = './../../../' + parts.join('/');
            } else {
                srcFiles[directory].push('/../../' + parts.join('/'));
            }
        }
    }

    grunt.registerTask('backendStep', 'build backend config files', function () {
        var srcFiles = {
            config: [],
            datasources: {}
        };
        var directories = ['config', 'datasources'];
        var dest = {
            config: '../backend/config/servicesMapping.json',
            datasources: '../backend/config/dataSourcesMapping.json'
        };
        directories.forEach(function (directory) {
            backendModules.forEach(function (module) {
                var moduleParts = module.split('/');
                var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
                var subModules = grunt.file.expand({
                    filter: 'isDirectory'
                }, module + '/src/Teknisa/' + moduleName + '/Modules/*');

                subModules.forEach(function (subModule) {
                    grunt.file.expand({
                        filter: 'isFile'
                    }, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
                        buildFileContent(srcFiles, filePath, directory);
                    });
                });

                grunt.file.expand({
                    filter: 'isFile'
                }, module + '/' + directory + '/**/*').forEach(function (filePath) {
                    buildFileContent(srcFiles, filePath, directory);
                });
            });
        });
        grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
        grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
    });

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');

    grunt.registerTask('default', ['jshint', 'uglify', 'concat', 'backendStep', 'buildEnvironment', 'buildAclStruct']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);
    grunt.registerTask('basic', ['uglify', 'concat:js', 'zhIdGenerator']);
};