function DCUtilities(templateManager, ScreenService, LocalVars, DCMessages, requestDataSourceEngine, Query, ZhPreferences, ZhFilterPreferences)
{
    this.localVar = LocalVars;
    this.messages = DCMessages;
    var self = this;
    this.setOutData = function(field, value) {
        var vField = field.valueField;
        if (value) {
            var matchRows = field.dataSource.data.filter(function (item) {
                return item[vField] == value;
            });
            if (matchRows.length > 0) {
                Object.keys(field.outData).forEach(function (key) {
                    field.widget.currentRow[field.outData[key]] = matchRows[0][key];
                });
            }
        }
    };

    this.fixWidgetToViewAfterSaveRow = function(widget) {
        widget.view();
        widget.template = widget.templateToView;
        widget.activate();
        widget.currentRow.__is_new = false;
        widget.currentRow.changed = false;
        widget.setCurrentRow(widget.currentRow);
    };

    this.genericDeleteEvent = function (widget){
        return widget.dataSource.remove([widget.currentRow]).then(function(data){
            return ScreenService.closeSwipe().then(function(){
                return widget.reload();
            });
        });
    };

    this.isNotEmptyArray = function(array) {
        return array.length > 0;
    };

    this.createPathsForMessage = function(paths, pathsString){
        paths.forEach(function (path) {
            pathsString += "<u>" + path + "</u><br/>";
        });
        return pathsString;
    };

    this.genericCancelEvent = function(widget, primaryProperty) {
        primaryProperty = primaryProperty ? primaryProperty :"ID";
        var primaryValue = widget.currentRow[primaryProperty];
        if (widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
            widget.reload();
        } else {
            widget.reload().then(function(){
                widget.view();
                return widget.dataSource.data.some(function (row) {
                    if (row[primaryProperty] == primaryValue) {
                        widget.currentRow = row;
                        return true;
                    }
                });
            });
        }
        templateManager.updateTemplate();
    };

    this.startFilter = function(widget, currentRow, isApplyFilter) {
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };
                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.setAppliedFilter(defaultFilter || filter);
                        if(defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else if(isApplyFilter) {
                ZhFilterPreferences.applyOriginalFilter(widget);
            } else {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        });
    };

    this.getRowInDataSource = function (dataSource, primaryProperty, primaryValue) {
        var currentRow = null;
        dataSource.data.some(function (row) {
            if (row[primaryProperty] == primaryValue) {
                currentRow = row;
                return true;
            }
        });
        return currentRow;
    };

    this.showMessageErrorOfBackEnd = function(response,errorMessages,errorMessageDefault,showMessageBackEndTrue) {
        var message = errorMessageDefault ? errorMessageDefault : this.messages.UNEXPECTED_ERROR;
        if (showMessageBackEndTrue) {
            if (response && response.data && response.data.error) {
                message = response.data.error.split("Exception:").pop();
            }
        } else {
            if (response && response.data && response.data.error) {
                var errorCode = response.data.error.split("Code:").pop();
                if (errorMessages && errorCode && errorMessages[errorCode]) {
                    message = errorMessages[errorCode] ? errorMessages[errorCode] : message;
                }
            }
        }
        return ScreenService.showMessage(message, 'ERROR');
    };

    this.showMessageBackEndTrue = function(response) {
        return self.showMessageErrorOfBackEnd(response,null,null,true);
    };

    this.fixBugReloadWidgetInViewMode = function(widget){
        widget.list();
        return widget.reload().then(function(){
            widget.view();
            widget.template = widget.templateToView;
        });
    };

    this.setDefaultValuesToSelectMultipleField = function(field,value,index){
        var data = this.findInDataSource(field,value,index);
        field.value(this.createArrayByDataSetAndIndex(data,field.valueField));
    };

    this.findInDataSource = function(field,values,index){
        return field.dataSource.data.filter(function(row){
            var found = false;
            values.forEach(function(value){
                if(row[index] == value){
                    found = true;
                }
            });
            return found;
        });
    };

    this.isAppliedFilter = function(widget){
        return !!widget.dcAppliedFilter;
    };

    this.setFilterInit = function (widget) {
        widget.dcAppliedFilter = true;
    };

    this.setDefaultFilterOnEnter = function(widget, isApplyFilter) {
        if (widget.inListMode()) {
            if (widget.devOpsAppliedFilter) {
                widget.reload();
            } else {
                var currentRow = widget.currentRowDefaultFilter || widget.currentRow;
                self.startFilter(widget, currentRow, isApplyFilter);
            }
        }
    };

    this.createArrayByDataSetAndIndex = function(dataSet,index){
        var array = [];
        dataSet.forEach(function(row){
            array.push(row[index]);
        });
        return array;
    };

    this.getWidgetCurrentRowAsChanged = function(widget){
        var row = widget.currentRow;
        row.changed = true;
        return row;
    };

    this.setDataSourceAfterSave = function(widget, filter, valueIndex){
        var widgetDataSource = widget.dataSource.data;
        return requestDataSourceEngine.getDataSource(widget.dataSource.name, filter).then(function (response) {
            var sponsorDataUpdated = response.dataset[widget.dataSource.name.split('/')[1]];
            var isRow = widgetDataSource.some(function(data, index){
                if(data[valueIndex] == sponsorDataUpdated[0][valueIndex]){
                    widgetDataSource[index] = sponsorDataUpdated[0];
                    return true;
                }
            });
            if(!isRow) {
                widgetDataSource.push(sponsorDataUpdated[0]);
            }
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('DCUtilities', DCUtilities);
});
